"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import Any, List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class FieldUpdateDocumentFieldsTypeDropdownRequest1(str, Enum):
    DROPDOWN = "DROPDOWN"


class FieldUpdateDocumentFieldsTypeDropdownRequest2(str, Enum):
    DROPDOWN = "dropdown"


class FieldUpdateDocumentFieldsValueDropdownTypedDict(TypedDict):
    value: str


class FieldUpdateDocumentFieldsValueDropdown(BaseModel):
    value: str


class FieldUpdateDocumentFieldsFieldMetaDropdownRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeDropdownRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueDropdownTypedDict]]
    default_value: NotRequired[str]


class FieldUpdateDocumentFieldsFieldMetaDropdownRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeDropdownRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueDropdown]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldUpdateDocumentFieldsFieldDropdownTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeDropdownRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaDropdownRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldDropdown(BaseModel):
    type: FieldUpdateDocumentFieldsTypeDropdownRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaDropdownRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeCheckboxRequest1(str, Enum):
    CHECKBOX = "CHECKBOX"


class FieldUpdateDocumentFieldsTypeCheckboxRequest2(str, Enum):
    CHECKBOX = "checkbox"


class FieldUpdateDocumentFieldsValueCheckboxTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateDocumentFieldsValueCheckbox(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateDocumentFieldsDirectionCheckbox(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateDocumentFieldsFieldMetaCheckboxRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeCheckboxRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueCheckboxTypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldUpdateDocumentFieldsDirectionCheckbox]


class FieldUpdateDocumentFieldsFieldMetaCheckboxRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeCheckboxRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueCheckbox]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldUpdateDocumentFieldsDirectionCheckbox] = (
        FieldUpdateDocumentFieldsDirectionCheckbox.VERTICAL
    )


class FieldUpdateDocumentFieldsFieldCheckboxTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeCheckboxRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaCheckboxRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldCheckbox(BaseModel):
    type: FieldUpdateDocumentFieldsTypeCheckboxRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaCheckboxRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeRadioRequest1(str, Enum):
    RADIO = "RADIO"


class FieldUpdateDocumentFieldsTypeRadioRequest2(str, Enum):
    RADIO = "radio"


class FieldUpdateDocumentFieldsValueRadioTypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateDocumentFieldsValueRadio(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateDocumentFieldsDirectionRadio(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateDocumentFieldsFieldMetaRadioRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeRadioRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueRadioTypedDict]]
    direction: NotRequired[FieldUpdateDocumentFieldsDirectionRadio]


class FieldUpdateDocumentFieldsFieldMetaRadioRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeRadioRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueRadio]] = None

    direction: Optional[FieldUpdateDocumentFieldsDirectionRadio] = (
        FieldUpdateDocumentFieldsDirectionRadio.VERTICAL
    )


class FieldUpdateDocumentFieldsFieldRadioTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeRadioRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaRadioRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldRadio(BaseModel):
    type: FieldUpdateDocumentFieldsTypeRadioRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaRadioRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeNumberRequest1(str, Enum):
    NUMBER = "NUMBER"


class FieldUpdateDocumentFieldsTypeNumberRequest2(str, Enum):
    NUMBER = "number"


class FieldUpdateDocumentFieldsTextAlignNumber(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsVerticalAlignNumber(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateDocumentFieldsFieldMetaNumberRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeNumberRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignNumber]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldUpdateDocumentFieldsVerticalAlignNumber]]


class FieldUpdateDocumentFieldsFieldMetaNumberRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeNumberRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignNumber],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateDocumentFieldsVerticalAlignNumber],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateDocumentFieldsFieldNumberTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeNumberRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaNumberRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldNumber(BaseModel):
    type: FieldUpdateDocumentFieldsTypeNumberRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaNumberRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeTextRequest1(str, Enum):
    TEXT = "TEXT"


class FieldUpdateDocumentFieldsTypeTextRequest2(str, Enum):
    TEXT = "text"


class FieldUpdateDocumentFieldsTextAlignText(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsVerticalAlignText(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateDocumentFieldsFieldMetaTextRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeTextRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignText]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[Nullable[FieldUpdateDocumentFieldsVerticalAlignText]]


class FieldUpdateDocumentFieldsFieldMetaTextRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeTextRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignText],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateDocumentFieldsVerticalAlignText],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateDocumentFieldsFieldTextTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeTextRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaTextRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldText(BaseModel):
    type: FieldUpdateDocumentFieldsTypeTextRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaTextRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeDateRequest1(str, Enum):
    DATE = "DATE"


class FieldUpdateDocumentFieldsTypeDateRequest2(str, Enum):
    DATE = "date"


class FieldUpdateDocumentFieldsTextAlignDate(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaDateRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeDateRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignDate]


class FieldUpdateDocumentFieldsFieldMetaDateRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeDateRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignDate],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldDateTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeDateRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaDateRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldDate(BaseModel):
    type: FieldUpdateDocumentFieldsTypeDateRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaDateRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeEmailRequest1(str, Enum):
    EMAIL = "EMAIL"


class FieldUpdateDocumentFieldsTypeEmailRequest2(str, Enum):
    EMAIL = "email"


class FieldUpdateDocumentFieldsTextAlignEmail(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaEmailRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeEmailRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignEmail]


class FieldUpdateDocumentFieldsFieldMetaEmailRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeEmailRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignEmail],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldEmailTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeEmailRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaEmailRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldEmail(BaseModel):
    type: FieldUpdateDocumentFieldsTypeEmailRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaEmailRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeNameRequest1(str, Enum):
    NAME = "NAME"


class FieldUpdateDocumentFieldsTypeNameRequest2(str, Enum):
    NAME = "name"


class FieldUpdateDocumentFieldsTextAlignName(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaNameRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeNameRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignName]


class FieldUpdateDocumentFieldsFieldMetaNameRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeNameRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignName],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldNameTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeNameRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaNameRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldName(BaseModel):
    type: FieldUpdateDocumentFieldsTypeNameRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaNameRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeInitialsRequest1(str, Enum):
    INITIALS = "INITIALS"


class FieldUpdateDocumentFieldsTypeInitialsRequest2(str, Enum):
    INITIALS = "initials"


class FieldUpdateDocumentFieldsTextAlignInitials(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaInitialsRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeInitialsRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignInitials]


class FieldUpdateDocumentFieldsFieldMetaInitialsRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeInitialsRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignInitials],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldInitialsTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeInitialsRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaInitialsRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldInitials(BaseModel):
    type: FieldUpdateDocumentFieldsTypeInitialsRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaInitialsRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeFreeSignature(str, Enum):
    FREE_SIGNATURE = "FREE_SIGNATURE"


class FieldUpdateDocumentFieldsFieldFreeSignatureTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeFreeSignature
    id: float
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldFreeSignature(BaseModel):
    type: FieldUpdateDocumentFieldsTypeFreeSignature

    id: float

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


class FieldUpdateDocumentFieldsTypeSignatureRequest1(str, Enum):
    SIGNATURE = "SIGNATURE"


class FieldUpdateDocumentFieldsTypeSignatureRequest2(str, Enum):
    SIGNATURE = "signature"


class FieldUpdateDocumentFieldsFieldMetaSignatureRequestTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeSignatureRequest2
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldUpdateDocumentFieldsFieldMetaSignatureRequest(BaseModel):
    type: FieldUpdateDocumentFieldsTypeSignatureRequest2

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


class FieldUpdateDocumentFieldsFieldSignatureTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsTypeSignatureRequest1
    id: float
    field_meta: NotRequired[FieldUpdateDocumentFieldsFieldMetaSignatureRequestTypedDict]
    page_number: NotRequired[float]
    page_x: NotRequired[float]
    page_y: NotRequired[float]
    width: NotRequired[float]
    height: NotRequired[float]


class FieldUpdateDocumentFieldsFieldSignature(BaseModel):
    type: FieldUpdateDocumentFieldsTypeSignatureRequest1

    id: float

    field_meta: Annotated[
        Optional[FieldUpdateDocumentFieldsFieldMetaSignatureRequest],
        pydantic.Field(alias="fieldMeta"),
    ] = None

    page_number: Annotated[Optional[float], pydantic.Field(alias="pageNumber")] = None

    page_x: Annotated[Optional[float], pydantic.Field(alias="pageX")] = None

    page_y: Annotated[Optional[float], pydantic.Field(alias="pageY")] = None

    width: Optional[float] = None

    height: Optional[float] = None


FieldUpdateDocumentFieldsFieldUnionTypedDict = TypeAliasType(
    "FieldUpdateDocumentFieldsFieldUnionTypedDict",
    Union[
        FieldUpdateDocumentFieldsFieldFreeSignatureTypedDict,
        FieldUpdateDocumentFieldsFieldSignatureTypedDict,
        FieldUpdateDocumentFieldsFieldInitialsTypedDict,
        FieldUpdateDocumentFieldsFieldNameTypedDict,
        FieldUpdateDocumentFieldsFieldEmailTypedDict,
        FieldUpdateDocumentFieldsFieldDateTypedDict,
        FieldUpdateDocumentFieldsFieldTextTypedDict,
        FieldUpdateDocumentFieldsFieldNumberTypedDict,
        FieldUpdateDocumentFieldsFieldRadioTypedDict,
        FieldUpdateDocumentFieldsFieldCheckboxTypedDict,
        FieldUpdateDocumentFieldsFieldDropdownTypedDict,
    ],
)


FieldUpdateDocumentFieldsFieldUnion = TypeAliasType(
    "FieldUpdateDocumentFieldsFieldUnion",
    Union[
        FieldUpdateDocumentFieldsFieldFreeSignature,
        FieldUpdateDocumentFieldsFieldSignature,
        FieldUpdateDocumentFieldsFieldInitials,
        FieldUpdateDocumentFieldsFieldName,
        FieldUpdateDocumentFieldsFieldEmail,
        FieldUpdateDocumentFieldsFieldDate,
        FieldUpdateDocumentFieldsFieldText,
        FieldUpdateDocumentFieldsFieldNumber,
        FieldUpdateDocumentFieldsFieldRadio,
        FieldUpdateDocumentFieldsFieldCheckbox,
        FieldUpdateDocumentFieldsFieldDropdown,
    ],
)


class FieldUpdateDocumentFieldsRequestTypedDict(TypedDict):
    document_id: float
    fields: List[FieldUpdateDocumentFieldsFieldUnionTypedDict]


class FieldUpdateDocumentFieldsRequest(BaseModel):
    document_id: Annotated[float, pydantic.Field(alias="documentId")]

    fields: List[FieldUpdateDocumentFieldsFieldUnion]


class FieldUpdateDocumentFieldsInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class FieldUpdateDocumentFieldsInternalServerErrorIssue(BaseModel):
    message: str


class FieldUpdateDocumentFieldsInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateDocumentFieldsInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateDocumentFieldsInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: FieldUpdateDocumentFieldsInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateDocumentFieldsInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateDocumentFieldsForbiddenIssueTypedDict(TypedDict):
    message: str


class FieldUpdateDocumentFieldsForbiddenIssue(BaseModel):
    message: str


class FieldUpdateDocumentFieldsForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateDocumentFieldsForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateDocumentFieldsForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: FieldUpdateDocumentFieldsForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateDocumentFieldsForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateDocumentFieldsUnauthorizedIssueTypedDict(TypedDict):
    message: str


class FieldUpdateDocumentFieldsUnauthorizedIssue(BaseModel):
    message: str


class FieldUpdateDocumentFieldsUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateDocumentFieldsUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateDocumentFieldsUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: FieldUpdateDocumentFieldsUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateDocumentFieldsUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateDocumentFieldsBadRequestIssueTypedDict(TypedDict):
    message: str


class FieldUpdateDocumentFieldsBadRequestIssue(BaseModel):
    message: str


class FieldUpdateDocumentFieldsBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FieldUpdateDocumentFieldsBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class FieldUpdateDocumentFieldsBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: FieldUpdateDocumentFieldsBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: FieldUpdateDocumentFieldsBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FieldUpdateDocumentFieldsTypeResponse(str, Enum):
    SIGNATURE = "SIGNATURE"
    FREE_SIGNATURE = "FREE_SIGNATURE"
    INITIALS = "INITIALS"
    NAME = "NAME"
    EMAIL = "EMAIL"
    DATE = "DATE"
    TEXT = "TEXT"
    NUMBER = "NUMBER"
    RADIO = "RADIO"
    CHECKBOX = "CHECKBOX"
    DROPDOWN = "DROPDOWN"


class FieldUpdateDocumentFieldsFieldMetaTypeDropdown(str, Enum):
    DROPDOWN = "dropdown"


class FieldUpdateDocumentFieldsValueResponse3TypedDict(TypedDict):
    value: str


class FieldUpdateDocumentFieldsValueResponse3(BaseModel):
    value: str


class FieldUpdateDocumentFieldsFieldMetaDropdownResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeDropdown
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueResponse3TypedDict]]
    default_value: NotRequired[str]


class FieldUpdateDocumentFieldsFieldMetaDropdownResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeDropdown

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueResponse3]] = None

    default_value: Annotated[Optional[str], pydantic.Field(alias="defaultValue")] = None


class FieldUpdateDocumentFieldsFieldMetaTypeCheckbox(str, Enum):
    CHECKBOX = "checkbox"


class FieldUpdateDocumentFieldsValueResponse2TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateDocumentFieldsValueResponse2(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateDocumentFieldsDirectionResponse2(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateDocumentFieldsFieldMetaCheckboxResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeCheckbox
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueResponse2TypedDict]]
    validation_rule: NotRequired[str]
    validation_length: NotRequired[float]
    direction: NotRequired[FieldUpdateDocumentFieldsDirectionResponse2]


class FieldUpdateDocumentFieldsFieldMetaCheckboxResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeCheckbox

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueResponse2]] = None

    validation_rule: Annotated[
        Optional[str], pydantic.Field(alias="validationRule")
    ] = None

    validation_length: Annotated[
        Optional[float], pydantic.Field(alias="validationLength")
    ] = None

    direction: Optional[FieldUpdateDocumentFieldsDirectionResponse2] = (
        FieldUpdateDocumentFieldsDirectionResponse2.VERTICAL
    )


class FieldUpdateDocumentFieldsFieldMetaTypeRadio(str, Enum):
    RADIO = "radio"


class FieldUpdateDocumentFieldsValueResponse1TypedDict(TypedDict):
    id: float
    checked: bool
    value: str


class FieldUpdateDocumentFieldsValueResponse1(BaseModel):
    id: float

    checked: bool

    value: str


class FieldUpdateDocumentFieldsDirectionResponse1(str, Enum):
    VERTICAL = "vertical"
    HORIZONTAL = "horizontal"


class FieldUpdateDocumentFieldsFieldMetaRadioResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeRadio
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    values: NotRequired[List[FieldUpdateDocumentFieldsValueResponse1TypedDict]]
    direction: NotRequired[FieldUpdateDocumentFieldsDirectionResponse1]


class FieldUpdateDocumentFieldsFieldMetaRadioResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeRadio

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    values: Optional[List[FieldUpdateDocumentFieldsValueResponse1]] = None

    direction: Optional[FieldUpdateDocumentFieldsDirectionResponse1] = (
        FieldUpdateDocumentFieldsDirectionResponse1.VERTICAL
    )


class FieldUpdateDocumentFieldsFieldMetaTypeNumber(str, Enum):
    NUMBER = "number"


class FieldUpdateDocumentFieldsTextAlignResponse6(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsVerticalAlignResponse2(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateDocumentFieldsFieldMetaNumberResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeNumber
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    number_format: NotRequired[Nullable[str]]
    value: NotRequired[str]
    min_value: NotRequired[Nullable[float]]
    max_value: NotRequired[Nullable[float]]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse6]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldUpdateDocumentFieldsVerticalAlignResponse2]
    ]


class FieldUpdateDocumentFieldsFieldMetaNumberResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeNumber

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    number_format: Annotated[
        OptionalNullable[str], pydantic.Field(alias="numberFormat")
    ] = UNSET

    value: Optional[str] = None

    min_value: Annotated[OptionalNullable[float], pydantic.Field(alias="minValue")] = (
        UNSET
    )

    max_value: Annotated[OptionalNullable[float], pydantic.Field(alias="maxValue")] = (
        UNSET
    )

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse6],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateDocumentFieldsVerticalAlignResponse2],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "numberFormat",
            "value",
            "minValue",
            "maxValue",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = [
            "numberFormat",
            "minValue",
            "maxValue",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateDocumentFieldsFieldMetaTypeText(str, Enum):
    TEXT = "text"


class FieldUpdateDocumentFieldsTextAlignResponse5(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsVerticalAlignResponse1(str, Enum):
    TOP = "top"
    MIDDLE = "middle"
    BOTTOM = "bottom"


class FieldUpdateDocumentFieldsFieldMetaTextResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeText
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text: NotRequired[str]
    character_limit: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse5]
    line_height: NotRequired[Nullable[float]]
    letter_spacing: NotRequired[Nullable[float]]
    vertical_align: NotRequired[
        Nullable[FieldUpdateDocumentFieldsVerticalAlignResponse1]
    ]


class FieldUpdateDocumentFieldsFieldMetaTextResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeText

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text: Optional[str] = None

    character_limit: Annotated[
        Optional[float], pydantic.Field(alias="characterLimit")
    ] = None

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse5],
        pydantic.Field(alias="textAlign"),
    ] = None

    line_height: Annotated[
        OptionalNullable[float], pydantic.Field(alias="lineHeight")
    ] = UNSET

    letter_spacing: Annotated[
        OptionalNullable[float], pydantic.Field(alias="letterSpacing")
    ] = UNSET

    vertical_align: Annotated[
        OptionalNullable[FieldUpdateDocumentFieldsVerticalAlignResponse1],
        pydantic.Field(alias="verticalAlign"),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "label",
            "placeholder",
            "required",
            "readOnly",
            "fontSize",
            "text",
            "characterLimit",
            "textAlign",
            "lineHeight",
            "letterSpacing",
            "verticalAlign",
        ]
        nullable_fields = ["lineHeight", "letterSpacing", "verticalAlign"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateDocumentFieldsFieldMetaTypeDate(str, Enum):
    DATE = "date"


class FieldUpdateDocumentFieldsTextAlignResponse4(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaDateResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeDate
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse4]


class FieldUpdateDocumentFieldsFieldMetaDateResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeDate

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse4],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldMetaTypeEmail(str, Enum):
    EMAIL = "email"


class FieldUpdateDocumentFieldsTextAlignResponse3(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaEmailResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeEmail
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse3]


class FieldUpdateDocumentFieldsFieldMetaEmailResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeEmail

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse3],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldMetaTypeName(str, Enum):
    NAME = "name"


class FieldUpdateDocumentFieldsTextAlignResponse2(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaNameResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeName
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse2]


class FieldUpdateDocumentFieldsFieldMetaNameResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeName

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse2],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldMetaTypeInitials(str, Enum):
    INITIALS = "initials"


class FieldUpdateDocumentFieldsTextAlignResponse1(str, Enum):
    LEFT = "left"
    CENTER = "center"
    RIGHT = "right"


class FieldUpdateDocumentFieldsFieldMetaInitialsResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeInitials
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]
    text_align: NotRequired[FieldUpdateDocumentFieldsTextAlignResponse1]


class FieldUpdateDocumentFieldsFieldMetaInitialsResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeInitials

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12

    text_align: Annotated[
        Optional[FieldUpdateDocumentFieldsTextAlignResponse1],
        pydantic.Field(alias="textAlign"),
    ] = None


class FieldUpdateDocumentFieldsFieldMetaTypeSignature(str, Enum):
    SIGNATURE = "signature"


class FieldUpdateDocumentFieldsFieldMetaSignatureResponseTypedDict(TypedDict):
    type: FieldUpdateDocumentFieldsFieldMetaTypeSignature
    label: NotRequired[str]
    placeholder: NotRequired[str]
    required: NotRequired[bool]
    read_only: NotRequired[bool]
    font_size: NotRequired[float]


class FieldUpdateDocumentFieldsFieldMetaSignatureResponse(BaseModel):
    type: FieldUpdateDocumentFieldsFieldMetaTypeSignature

    label: Optional[str] = None

    placeholder: Optional[str] = None

    required: Optional[bool] = None

    read_only: Annotated[Optional[bool], pydantic.Field(alias="readOnly")] = None

    font_size: Annotated[Optional[float], pydantic.Field(alias="fontSize")] = 12


FieldUpdateDocumentFieldsFieldMetaUnionTypedDict = TypeAliasType(
    "FieldUpdateDocumentFieldsFieldMetaUnionTypedDict",
    Union[
        FieldUpdateDocumentFieldsFieldMetaSignatureResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaInitialsResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaNameResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaEmailResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaDateResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaRadioResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaDropdownResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaCheckboxResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaTextResponseTypedDict,
        FieldUpdateDocumentFieldsFieldMetaNumberResponseTypedDict,
    ],
)


FieldUpdateDocumentFieldsFieldMetaUnion = TypeAliasType(
    "FieldUpdateDocumentFieldsFieldMetaUnion",
    Union[
        FieldUpdateDocumentFieldsFieldMetaSignatureResponse,
        FieldUpdateDocumentFieldsFieldMetaInitialsResponse,
        FieldUpdateDocumentFieldsFieldMetaNameResponse,
        FieldUpdateDocumentFieldsFieldMetaEmailResponse,
        FieldUpdateDocumentFieldsFieldMetaDateResponse,
        FieldUpdateDocumentFieldsFieldMetaRadioResponse,
        FieldUpdateDocumentFieldsFieldMetaDropdownResponse,
        FieldUpdateDocumentFieldsFieldMetaCheckboxResponse,
        FieldUpdateDocumentFieldsFieldMetaTextResponse,
        FieldUpdateDocumentFieldsFieldMetaNumberResponse,
    ],
)


class FieldUpdateDocumentFieldsFieldResponseTypedDict(TypedDict):
    envelope_id: str
    envelope_item_id: str
    type: FieldUpdateDocumentFieldsTypeResponse
    id: float
    secondary_id: str
    recipient_id: float
    page: float
    position_x: Any
    position_y: Any
    width: Any
    height: Any
    custom_text: str
    inserted: bool
    field_meta: Nullable[FieldUpdateDocumentFieldsFieldMetaUnionTypedDict]
    document_id: NotRequired[Nullable[float]]
    template_id: NotRequired[Nullable[float]]


class FieldUpdateDocumentFieldsFieldResponse(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    envelope_item_id: Annotated[str, pydantic.Field(alias="envelopeItemId")]

    type: FieldUpdateDocumentFieldsTypeResponse

    id: float

    secondary_id: Annotated[str, pydantic.Field(alias="secondaryId")]

    recipient_id: Annotated[float, pydantic.Field(alias="recipientId")]

    page: float

    position_x: Annotated[Any, pydantic.Field(alias="positionX")]

    position_y: Annotated[Any, pydantic.Field(alias="positionY")]

    width: Any

    height: Any

    custom_text: Annotated[str, pydantic.Field(alias="customText")]

    inserted: bool

    field_meta: Annotated[
        Nullable[FieldUpdateDocumentFieldsFieldMetaUnion],
        pydantic.Field(alias="fieldMeta"),
    ]

    document_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="documentId")
    ] = UNSET

    template_id: Annotated[
        OptionalNullable[float], pydantic.Field(alias="templateId")
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["documentId", "templateId"]
        nullable_fields = ["fieldMeta", "documentId", "templateId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class FieldUpdateDocumentFieldsResponseTypedDict(TypedDict):
    r"""Successful response"""

    fields: List[FieldUpdateDocumentFieldsFieldResponseTypedDict]


class FieldUpdateDocumentFieldsResponse(BaseModel):
    r"""Successful response"""

    fields: List[FieldUpdateDocumentFieldsFieldResponse]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class TemplateDeleteTemplateRequestTypedDict(TypedDict):
    template_id: float


class TemplateDeleteTemplateRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]


class TemplateDeleteTemplateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateInternalServerErrorIssue(BaseModel):
    message: str


class TemplateDeleteTemplateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateDeleteTemplateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateForbiddenIssue(BaseModel):
    message: str


class TemplateDeleteTemplateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateDeleteTemplateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateUnauthorizedIssue(BaseModel):
    message: str


class TemplateDeleteTemplateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateDeleteTemplateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateBadRequestIssue(BaseModel):
    message: str


class TemplateDeleteTemplateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateDeleteTemplateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class TemplateDeleteTemplateResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from documenso_sdk import models, utils
from documenso_sdk._hooks import HookContext
from documenso_sdk.types import OptionalNullable, UNSET
from documenso_sdk.utils import get_security_from_env
from documenso_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class TemplateSDK(BaseSDK):
    def template_create_template_temporary(
        self,
        *,
        title: str,
        folder_id: Optional[str] = None,
        external_id: OptionalNullable[str] = UNSET,
        visibility: Optional[
            models.TemplateCreateTemplateTemporaryVisibilityRequest
        ] = None,
        global_access_auth: Optional[
            List[models.TemplateCreateTemplateTemporaryGlobalAccessAuthRequest]
        ] = None,
        global_action_auth: Optional[
            List[models.TemplateCreateTemplateTemporaryGlobalActionAuthRequest]
        ] = None,
        public_title: Optional[str] = None,
        public_description: Optional[str] = None,
        type_: Optional[models.TemplateCreateTemplateTemporaryTypeRequest] = None,
        meta: Optional[
            Union[
                models.TemplateCreateTemplateTemporaryMeta,
                models.TemplateCreateTemplateTemporaryMetaTypedDict,
            ]
        ] = None,
        attachments: Optional[
            Union[
                List[models.TemplateCreateTemplateTemporaryAttachment],
                List[models.TemplateCreateTemplateTemporaryAttachmentTypedDict],
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TemplateCreateTemplateTemporaryResponse:
        r"""Create template

        You will need to upload the PDF to the provided URL returned. Note: Once V2 API is released, this will be removed since we will allow direct uploads, instead of using an upload URL.

        :param title:
        :param folder_id:
        :param external_id:
        :param visibility:
        :param global_access_auth:
        :param global_action_auth:
        :param public_title:
        :param public_description:
        :param type:
        :param meta:
        :param attachments:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TemplateCreateTemplateTemporaryRequest(
            title=title,
            folder_id=folder_id,
            external_id=external_id,
            visibility=visibility,
            global_access_auth=global_access_auth,
            global_action_auth=global_action_auth,
            public_title=public_title,
            public_description=public_description,
            type=type_,
            meta=utils.get_pydantic_model(
                meta, Optional[models.TemplateCreateTemplateTemporaryMeta]
            ),
            attachments=utils.get_pydantic_model(
                attachments,
                Optional[List[models.TemplateCreateTemplateTemporaryAttachment]],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/template/create/beta",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.TemplateCreateTemplateTemporaryRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="template-createTemplateTemporary",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryBadRequestErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryUnauthorizedErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryForbiddenErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryInternalServerErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def template_create_template_temporary_async(
        self,
        *,
        title: str,
        folder_id: Optional[str] = None,
        external_id: OptionalNullable[str] = UNSET,
        visibility: Optional[
            models.TemplateCreateTemplateTemporaryVisibilityRequest
        ] = None,
        global_access_auth: Optional[
            List[models.TemplateCreateTemplateTemporaryGlobalAccessAuthRequest]
        ] = None,
        global_action_auth: Optional[
            List[models.TemplateCreateTemplateTemporaryGlobalActionAuthRequest]
        ] = None,
        public_title: Optional[str] = None,
        public_description: Optional[str] = None,
        type_: Optional[models.TemplateCreateTemplateTemporaryTypeRequest] = None,
        meta: Optional[
            Union[
                models.TemplateCreateTemplateTemporaryMeta,
                models.TemplateCreateTemplateTemporaryMetaTypedDict,
            ]
        ] = None,
        attachments: Optional[
            Union[
                List[models.TemplateCreateTemplateTemporaryAttachment],
                List[models.TemplateCreateTemplateTemporaryAttachmentTypedDict],
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TemplateCreateTemplateTemporaryResponse:
        r"""Create template

        You will need to upload the PDF to the provided URL returned. Note: Once V2 API is released, this will be removed since we will allow direct uploads, instead of using an upload URL.

        :param title:
        :param folder_id:
        :param external_id:
        :param visibility:
        :param global_access_auth:
        :param global_action_auth:
        :param public_title:
        :param public_description:
        :param type:
        :param meta:
        :param attachments:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.TemplateCreateTemplateTemporaryRequest(
            title=title,
            folder_id=folder_id,
            external_id=external_id,
            visibility=visibility,
            global_access_auth=global_access_auth,
            global_action_auth=global_action_auth,
            public_title=public_title,
            public_description=public_description,
            type=type_,
            meta=utils.get_pydantic_model(
                meta, Optional[models.TemplateCreateTemplateTemporaryMeta]
            ),
            attachments=utils.get_pydantic_model(
                attachments,
                Optional[List[models.TemplateCreateTemplateTemporaryAttachment]],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/template/create/beta",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.TemplateCreateTemplateTemporaryRequest,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="template-createTemplateTemporary",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryResponse, http_res
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryBadRequestErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryBadRequestError(
                response_data, http_res
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryUnauthorizedErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryUnauthorizedError(
                response_data, http_res
            )
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryForbiddenErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryForbiddenError(
                response_data, http_res
            )
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.TemplateCreateTemplateTemporaryInternalServerErrorData, http_res
            )
            raise models.TemplateCreateTemplateTemporaryInternalServerError(
                response_data, http_res
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
from typing import List, Optional
from typing_extensions import TypedDict


class DocumentAttachmentDeleteRequestTypedDict(TypedDict):
    id: str


class DocumentAttachmentDeleteRequest(BaseModel):
    id: str


class DocumentAttachmentDeleteInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class DocumentAttachmentDeleteInternalServerErrorIssue(BaseModel):
    message: str


class DocumentAttachmentDeleteInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentAttachmentDeleteInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentAttachmentDeleteInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: DocumentAttachmentDeleteInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentAttachmentDeleteInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentAttachmentDeleteForbiddenIssueTypedDict(TypedDict):
    message: str


class DocumentAttachmentDeleteForbiddenIssue(BaseModel):
    message: str


class DocumentAttachmentDeleteForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentAttachmentDeleteForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentAttachmentDeleteForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: DocumentAttachmentDeleteForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentAttachmentDeleteForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentAttachmentDeleteUnauthorizedIssueTypedDict(TypedDict):
    message: str


class DocumentAttachmentDeleteUnauthorizedIssue(BaseModel):
    message: str


class DocumentAttachmentDeleteUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentAttachmentDeleteUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentAttachmentDeleteUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: DocumentAttachmentDeleteUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentAttachmentDeleteUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentAttachmentDeleteBadRequestIssueTypedDict(TypedDict):
    message: str


class DocumentAttachmentDeleteBadRequestIssue(BaseModel):
    message: str


class DocumentAttachmentDeleteBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[DocumentAttachmentDeleteBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class DocumentAttachmentDeleteBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: DocumentAttachmentDeleteBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: DocumentAttachmentDeleteBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class DocumentAttachmentDeleteResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class DocumentAttachmentDeleteResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from documenso_sdk import models, utils
from documenso_sdk._hooks import HookContext
from documenso_sdk.types import OptionalNullable, UNSET
from documenso_sdk.utils import get_security_from_env
from documenso_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class Folders(BaseSDK):
    def find(
        self,
        *,
        query: Optional[str] = None,
        page: Optional[float] = None,
        per_page: Optional[float] = None,
        parent_id: Optional[str] = None,
        type_: Optional[models.FolderFindFoldersQueryParamType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderFindFoldersResponse:
        r"""Find folders

        Find folders based on a search criteria

        :param query: The search query.
        :param page: The pagination page number, starts at 1.
        :param per_page: The number of items per page.
        :param parent_id:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderFindFoldersRequest(
            query=query,
            page=page,
            per_page=per_page,
            parent_id=parent_id,
            type=type_,
        )

        req = self._build_request(
            method="GET",
            path="/folder",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-findFolders",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderFindFoldersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersBadRequestErrorData, http_res
            )
            raise models.FolderFindFoldersBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersUnauthorizedErrorData, http_res
            )
            raise models.FolderFindFoldersUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersForbiddenErrorData, http_res
            )
            raise models.FolderFindFoldersForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersNotFoundErrorData, http_res
            )
            raise models.FolderFindFoldersNotFoundError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersInternalServerErrorData, http_res
            )
            raise models.FolderFindFoldersInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def find_async(
        self,
        *,
        query: Optional[str] = None,
        page: Optional[float] = None,
        per_page: Optional[float] = None,
        parent_id: Optional[str] = None,
        type_: Optional[models.FolderFindFoldersQueryParamType] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderFindFoldersResponse:
        r"""Find folders

        Find folders based on a search criteria

        :param query: The search query.
        :param page: The pagination page number, starts at 1.
        :param per_page: The number of items per page.
        :param parent_id:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderFindFoldersRequest(
            query=query,
            page=page,
            per_page=per_page,
            parent_id=parent_id,
            type=type_,
        )

        req = self._build_request_async(
            method="GET",
            path="/folder",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-findFolders",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderFindFoldersResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersBadRequestErrorData, http_res
            )
            raise models.FolderFindFoldersBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersUnauthorizedErrorData, http_res
            )
            raise models.FolderFindFoldersUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersForbiddenErrorData, http_res
            )
            raise models.FolderFindFoldersForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersNotFoundErrorData, http_res
            )
            raise models.FolderFindFoldersNotFoundError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderFindFoldersInternalServerErrorData, http_res
            )
            raise models.FolderFindFoldersInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        parent_id: Optional[str] = None,
        type_: Optional[models.FolderCreateFolderTypeRequest] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderCreateFolderResponse:
        r"""Create new folder

        Creates a new folder in your team

        :param name:
        :param parent_id:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderCreateFolderRequest(
            name=name,
            parent_id=parent_id,
            type=type_,
        )

        req = self._build_request(
            method="POST",
            path="/folder/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderCreateFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-createFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderCreateFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderBadRequestErrorData, http_res
            )
            raise models.FolderCreateFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderCreateFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderForbiddenErrorData, http_res
            )
            raise models.FolderCreateFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderInternalServerErrorData, http_res
            )
            raise models.FolderCreateFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        parent_id: Optional[str] = None,
        type_: Optional[models.FolderCreateFolderTypeRequest] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderCreateFolderResponse:
        r"""Create new folder

        Creates a new folder in your team

        :param name:
        :param parent_id:
        :param type:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderCreateFolderRequest(
            name=name,
            parent_id=parent_id,
            type=type_,
        )

        req = self._build_request_async(
            method="POST",
            path="/folder/create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderCreateFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-createFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderCreateFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderBadRequestErrorData, http_res
            )
            raise models.FolderCreateFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderCreateFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderForbiddenErrorData, http_res
            )
            raise models.FolderCreateFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderCreateFolderInternalServerErrorData, http_res
            )
            raise models.FolderCreateFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        folder_id: str,
        data: Union[
            models.FolderUpdateFolderData, models.FolderUpdateFolderDataTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderUpdateFolderResponse:
        r"""Update folder

        Updates an existing folder

        :param folder_id:
        :param data:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderUpdateFolderRequest(
            folder_id=folder_id,
            data=utils.get_pydantic_model(data, models.FolderUpdateFolderData),
        )

        req = self._build_request(
            method="POST",
            path="/folder/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderUpdateFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-updateFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderUpdateFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderBadRequestErrorData, http_res
            )
            raise models.FolderUpdateFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderUpdateFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderForbiddenErrorData, http_res
            )
            raise models.FolderUpdateFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderInternalServerErrorData, http_res
            )
            raise models.FolderUpdateFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        folder_id: str,
        data: Union[
            models.FolderUpdateFolderData, models.FolderUpdateFolderDataTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderUpdateFolderResponse:
        r"""Update folder

        Updates an existing folder

        :param folder_id:
        :param data:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderUpdateFolderRequest(
            folder_id=folder_id,
            data=utils.get_pydantic_model(data, models.FolderUpdateFolderData),
        )

        req = self._build_request_async(
            method="POST",
            path="/folder/update",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderUpdateFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-updateFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderUpdateFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderBadRequestErrorData, http_res
            )
            raise models.FolderUpdateFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderUpdateFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderForbiddenErrorData, http_res
            )
            raise models.FolderUpdateFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderUpdateFolderInternalServerErrorData, http_res
            )
            raise models.FolderUpdateFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        folder_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderDeleteFolderResponse:
        r"""Delete folder

        Deletes an existing folder

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderDeleteFolderRequest(
            folder_id=folder_id,
        )

        req = self._build_request(
            method="POST",
            path="/folder/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderDeleteFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-deleteFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderDeleteFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderBadRequestErrorData, http_res
            )
            raise models.FolderDeleteFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderDeleteFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderForbiddenErrorData, http_res
            )
            raise models.FolderDeleteFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderInternalServerErrorData, http_res
            )
            raise models.FolderDeleteFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        folder_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FolderDeleteFolderResponse:
        r"""Delete folder

        Deletes an existing folder

        :param folder_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.FolderDeleteFolderRequest(
            folder_id=folder_id,
        )

        req = self._build_request_async(
            method="POST",
            path="/folder/delete",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", models.FolderDeleteFolderRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="folder-deleteFolder",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.FolderDeleteFolderResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderBadRequestErrorData, http_res
            )
            raise models.FolderDeleteFolderBadRequestError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderUnauthorizedErrorData, http_res
            )
            raise models.FolderDeleteFolderUnauthorizedError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderForbiddenErrorData, http_res
            )
            raise models.FolderDeleteFolderForbiddenError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                models.FolderDeleteFolderInternalServerErrorData, http_res
            )
            raise models.FolderDeleteFolderInternalServerError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class TemplateDeleteTemplateDirectLinkRequestTypedDict(TypedDict):
    template_id: float


class TemplateDeleteTemplateDirectLinkRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]


class TemplateDeleteTemplateDirectLinkInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateDirectLinkInternalServerErrorIssue(BaseModel):
    message: str


class TemplateDeleteTemplateDirectLinkInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateDirectLinkInternalServerErrorIssue]] = (
        None
    )


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateDirectLinkInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateDeleteTemplateDirectLinkInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateDirectLinkInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateDirectLinkForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateDirectLinkForbiddenIssue(BaseModel):
    message: str


class TemplateDeleteTemplateDirectLinkForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateDirectLinkForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateDirectLinkForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateDeleteTemplateDirectLinkForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateDirectLinkForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateDirectLinkUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateDirectLinkUnauthorizedIssue(BaseModel):
    message: str


class TemplateDeleteTemplateDirectLinkUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateDirectLinkUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateDirectLinkUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateDeleteTemplateDirectLinkUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateDirectLinkUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateDirectLinkBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateDeleteTemplateDirectLinkBadRequestIssue(BaseModel):
    message: str


class TemplateDeleteTemplateDirectLinkBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDeleteTemplateDirectLinkBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDeleteTemplateDirectLinkBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateDeleteTemplateDirectLinkBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDeleteTemplateDirectLinkBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDeleteTemplateDirectLinkResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class TemplateDeleteTemplateDirectLinkResponse(BaseModel):
    r"""Successful response"""

    success: bool



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class EnvelopeAttachmentCreateDataTypedDict(TypedDict):
    label: str
    data: str


class EnvelopeAttachmentCreateData(BaseModel):
    label: str

    data: str


class EnvelopeAttachmentCreateRequestTypedDict(TypedDict):
    envelope_id: str
    data: EnvelopeAttachmentCreateDataTypedDict


class EnvelopeAttachmentCreateRequest(BaseModel):
    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    data: EnvelopeAttachmentCreateData


class EnvelopeAttachmentCreateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentCreateInternalServerErrorIssue(BaseModel):
    message: str


class EnvelopeAttachmentCreateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentCreateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentCreateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: EnvelopeAttachmentCreateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentCreateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentCreateForbiddenIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentCreateForbiddenIssue(BaseModel):
    message: str


class EnvelopeAttachmentCreateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentCreateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentCreateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: EnvelopeAttachmentCreateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentCreateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentCreateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentCreateUnauthorizedIssue(BaseModel):
    message: str


class EnvelopeAttachmentCreateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentCreateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentCreateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: EnvelopeAttachmentCreateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentCreateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentCreateBadRequestIssueTypedDict(TypedDict):
    message: str


class EnvelopeAttachmentCreateBadRequestIssue(BaseModel):
    message: str


class EnvelopeAttachmentCreateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[EnvelopeAttachmentCreateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class EnvelopeAttachmentCreateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: EnvelopeAttachmentCreateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: EnvelopeAttachmentCreateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class EnvelopeAttachmentCreateResponseTypedDict(TypedDict):
    r"""Successful response"""

    id: str


class EnvelopeAttachmentCreateResponse(BaseModel):
    r"""Successful response"""

    id: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel, Nullable, UNSET_SENTINEL
from enum import Enum
import httpx
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TemplateDuplicateTemplateRequestTypedDict(TypedDict):
    template_id: float


class TemplateDuplicateTemplateRequest(BaseModel):
    template_id: Annotated[float, pydantic.Field(alias="templateId")]


class TemplateDuplicateTemplateInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class TemplateDuplicateTemplateInternalServerErrorIssue(BaseModel):
    message: str


class TemplateDuplicateTemplateInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDuplicateTemplateInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDuplicateTemplateInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: TemplateDuplicateTemplateInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDuplicateTemplateInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDuplicateTemplateForbiddenIssueTypedDict(TypedDict):
    message: str


class TemplateDuplicateTemplateForbiddenIssue(BaseModel):
    message: str


class TemplateDuplicateTemplateForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDuplicateTemplateForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDuplicateTemplateForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: TemplateDuplicateTemplateForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDuplicateTemplateForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDuplicateTemplateUnauthorizedIssueTypedDict(TypedDict):
    message: str


class TemplateDuplicateTemplateUnauthorizedIssue(BaseModel):
    message: str


class TemplateDuplicateTemplateUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDuplicateTemplateUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDuplicateTemplateUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: TemplateDuplicateTemplateUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDuplicateTemplateUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDuplicateTemplateBadRequestIssueTypedDict(TypedDict):
    message: str


class TemplateDuplicateTemplateBadRequestIssue(BaseModel):
    message: str


class TemplateDuplicateTemplateBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[TemplateDuplicateTemplateBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class TemplateDuplicateTemplateBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: TemplateDuplicateTemplateBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: TemplateDuplicateTemplateBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class TemplateDuplicateTemplateType(str, Enum):
    PUBLIC = "PUBLIC"
    PRIVATE = "PRIVATE"


class TemplateDuplicateTemplateVisibility(str, Enum):
    EVERYONE = "EVERYONE"
    MANAGER_AND_ABOVE = "MANAGER_AND_ABOVE"
    ADMIN = "ADMIN"


class TemplateDuplicateTemplateGlobalAccessAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"


class TemplateDuplicateTemplateGlobalActionAuth(str, Enum):
    ACCOUNT = "ACCOUNT"
    PASSKEY = "PASSKEY"
    TWO_FACTOR_AUTH = "TWO_FACTOR_AUTH"
    PASSWORD = "PASSWORD"


class TemplateDuplicateTemplateAuthOptionsTypedDict(TypedDict):
    global_access_auth: List[TemplateDuplicateTemplateGlobalAccessAuth]
    global_action_auth: List[TemplateDuplicateTemplateGlobalActionAuth]


class TemplateDuplicateTemplateAuthOptions(BaseModel):
    global_access_auth: Annotated[
        List[TemplateDuplicateTemplateGlobalAccessAuth],
        pydantic.Field(alias="globalAccessAuth"),
    ]

    global_action_auth: Annotated[
        List[TemplateDuplicateTemplateGlobalActionAuth],
        pydantic.Field(alias="globalActionAuth"),
    ]


class TemplateDuplicateTemplateResponseTypedDict(TypedDict):
    r"""Successful response"""

    type: TemplateDuplicateTemplateType
    visibility: TemplateDuplicateTemplateVisibility
    id: float
    external_id: Nullable[str]
    title: str
    user_id: float
    team_id: float
    auth_options: Nullable[TemplateDuplicateTemplateAuthOptionsTypedDict]
    created_at: str
    updated_at: str
    public_title: str
    public_description: str
    folder_id: Nullable[str]
    use_legacy_field_insertion: bool
    envelope_id: str
    template_document_data_id: NotRequired[str]


class TemplateDuplicateTemplateResponse(BaseModel):
    r"""Successful response"""

    type: TemplateDuplicateTemplateType

    visibility: TemplateDuplicateTemplateVisibility

    id: float

    external_id: Annotated[Nullable[str], pydantic.Field(alias="externalId")]

    title: str

    user_id: Annotated[float, pydantic.Field(alias="userId")]

    team_id: Annotated[float, pydantic.Field(alias="teamId")]

    auth_options: Annotated[
        Nullable[TemplateDuplicateTemplateAuthOptions],
        pydantic.Field(alias="authOptions"),
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]

    updated_at: Annotated[str, pydantic.Field(alias="updatedAt")]

    public_title: Annotated[str, pydantic.Field(alias="publicTitle")]

    public_description: Annotated[str, pydantic.Field(alias="publicDescription")]

    folder_id: Annotated[Nullable[str], pydantic.Field(alias="folderId")]

    use_legacy_field_insertion: Annotated[
        bool, pydantic.Field(alias="useLegacyFieldInsertion")
    ]

    envelope_id: Annotated[str, pydantic.Field(alias="envelopeId")]

    template_document_data_id: Annotated[
        Optional[str], pydantic.Field(alias="templateDocumentDataId")
    ] = ""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["templateDocumentDataId"]
        nullable_fields = ["externalId", "authOptions", "folderId"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
from documenso_sdk.models import DocumensoError
from documenso_sdk.types import BaseModel
import httpx
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, TypedDict


class FolderDeleteFolderRequestTypedDict(TypedDict):
    folder_id: str


class FolderDeleteFolderRequest(BaseModel):
    folder_id: Annotated[str, pydantic.Field(alias="folderId")]


class FolderDeleteFolderInternalServerErrorIssueTypedDict(TypedDict):
    message: str


class FolderDeleteFolderInternalServerErrorIssue(BaseModel):
    message: str


class FolderDeleteFolderInternalServerErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FolderDeleteFolderInternalServerErrorIssue]] = None


@dataclass(unsafe_hash=True)
class FolderDeleteFolderInternalServerError(DocumensoError):
    r"""Internal server error"""

    data: FolderDeleteFolderInternalServerErrorData = field(hash=False)

    def __init__(
        self,
        data: FolderDeleteFolderInternalServerErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FolderDeleteFolderForbiddenIssueTypedDict(TypedDict):
    message: str


class FolderDeleteFolderForbiddenIssue(BaseModel):
    message: str


class FolderDeleteFolderForbiddenErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FolderDeleteFolderForbiddenIssue]] = None


@dataclass(unsafe_hash=True)
class FolderDeleteFolderForbiddenError(DocumensoError):
    r"""Insufficient access"""

    data: FolderDeleteFolderForbiddenErrorData = field(hash=False)

    def __init__(
        self,
        data: FolderDeleteFolderForbiddenErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FolderDeleteFolderUnauthorizedIssueTypedDict(TypedDict):
    message: str


class FolderDeleteFolderUnauthorizedIssue(BaseModel):
    message: str


class FolderDeleteFolderUnauthorizedErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FolderDeleteFolderUnauthorizedIssue]] = None


@dataclass(unsafe_hash=True)
class FolderDeleteFolderUnauthorizedError(DocumensoError):
    r"""Authorization not provided"""

    data: FolderDeleteFolderUnauthorizedErrorData = field(hash=False)

    def __init__(
        self,
        data: FolderDeleteFolderUnauthorizedErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FolderDeleteFolderBadRequestIssueTypedDict(TypedDict):
    message: str


class FolderDeleteFolderBadRequestIssue(BaseModel):
    message: str


class FolderDeleteFolderBadRequestErrorData(BaseModel):
    message: str

    code: str

    issues: Optional[List[FolderDeleteFolderBadRequestIssue]] = None


@dataclass(unsafe_hash=True)
class FolderDeleteFolderBadRequestError(DocumensoError):
    r"""Invalid input data"""

    data: FolderDeleteFolderBadRequestErrorData = field(hash=False)

    def __init__(
        self,
        data: FolderDeleteFolderBadRequestErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


class FolderDeleteFolderResponseTypedDict(TypedDict):
    r"""Successful response"""

    success: bool


class FolderDeleteFolderResponse(BaseModel):
    r"""Successful response"""

    success: bool
